

eng_unitsD = {} # key:engineering param name, value: Engineering units
eng_unitsD["A"] = "ft/sec"
eng_unitsD["Ac"] = "ft/sec"
eng_unitsD["C"] = "BTU/ft-hr-R"
eng_unitsD["Cc"] = "BTU/ft-hr-R"
eng_unitsD["Cond"] = "BTU/ft-hr-R"
eng_unitsD["Cp"] = "BTU/lbm degR"
eng_unitsD["Cv"] = "BTU/lbm degR"
eng_unitsD["D"] = "lbm/cu ft"
eng_unitsD["Dc"] = "lbm/cu ft"
eng_unitsD["Dmax"] = "lbm/cu ft"
eng_unitsD["Dmin"] = "lbm/cu ft"
eng_unitsD["E"] = "BTU/lbm"
eng_unitsD["Ec"] = "BTU/lbm"
eng_unitsD["g"] = "Cp/Cv (-)"
eng_unitsD["H"] = "BTU/lbm"
eng_unitsD["Hc"] = "BTU/lbm"
eng_unitsD["MW"] = "lbm/lbmmole"
eng_unitsD["P"] = "psia"
eng_unitsD["Pc"] = "psia"
eng_unitsD["Pref"] = "psia"
eng_unitsD["Psat"] = "psia"
eng_unitsD["Q"] = ""
eng_unitsD["rho"] = "lbm/cu in"
eng_unitsD["S"] = "BTU/lbm degR"
eng_unitsD["Sc"] = "BTU/lbm degR"
eng_unitsD["T"] = "degR"
eng_unitsD["Tc"] = "degR"
eng_unitsD["Tmax"] = "degR"
eng_unitsD["Tmin"] = "degR"
eng_unitsD["Tnbp"] = "degR"
eng_unitsD["Tref"] = "degR"
eng_unitsD["Tsat"] = "degR"
eng_unitsD["U"] = "BTU/lbm" # U is SI for E
eng_unitsD["V"] = "[1.0E5 * lbm/ft-sec]"
eng_unitsD["Vc"] = "[1.0E5 * lbm/ft-sec]"
eng_unitsD["Visc"] = "[1.0E5 * lbm/ft-sec]"
eng_unitsD["Z"] = ""
eng_unitsD["Zc"] = ""
# eng_unitsD["xxx"] = "xxx"
# eng_unitsD["xxx"] = "xxx"
# eng_unitsD["xxx"] = "xxx"
# eng_unitsD["xxx"] = "xxx"
# eng_unitsD["xxx"] = "xxx"
# eng_unitsD["xxx"] = "xxx"


# coolprop units from: http://www.coolprop.org/coolprop/HighLevelAPI.html#fluid-information
si_unitsD = {} # key:coolprop param name, value: SI units
si_unitsD["A"] = "m/s"
si_unitsD["ACENTRIC"] = ""
si_unitsD["acentric"] = ""
si_unitsD["ALPHA0"] = ""
si_unitsD["alpha0"] = ""
si_unitsD["ALPHAR"] = ""
si_unitsD["alphar"] = ""
si_unitsD["BVIRIAL"] = ""
si_unitsD["Bvirial"] = ""
si_unitsD["C"] = "J/kg/K"
si_unitsD["CONDUCTIVITY"] = "W/m/K"
si_unitsD["conductivity"] = "W/m/K"
si_unitsD["CP0MASS"] = "J/kg/K"
si_unitsD["Cp0mass"] = "J/kg/K"
si_unitsD["CP0MOLAR"] = "J/mol/K"
si_unitsD["Cp0molar"] = "J/mol/K"
si_unitsD["CPMASS"] = "J/kg/K"
si_unitsD["Cpmass"] = "J/kg/K"
si_unitsD["CPMOLAR"] = "J/mol/K"
si_unitsD["Cpmolar"] = "J/mol/K"
si_unitsD["CVIRIAL"] = ""
si_unitsD["Cvirial"] = ""
si_unitsD["CVMASS"] = "J/kg/K"
si_unitsD["Cvmass"] = "J/kg/K"
si_unitsD["CVMOLAR"] = "J/mol/K"
si_unitsD["Cvmolar"] = "J/mol/K"
si_unitsD["D"] = "kg/m^3"
si_unitsD["D2ALPHA0_DDELTA2_CONSTTAU"] = ""
si_unitsD["d2alpha0_ddelta2_consttau"] = ""
si_unitsD["D3ALPHA0_DDELTA3_CONSTTAU"] = ""
si_unitsD["d3alpha0_ddelta3_consttau"] = ""
si_unitsD["DALPHA0_DDELTA_CONSTTAU"] = ""
si_unitsD["dalpha0_ddelta_consttau"] = ""
si_unitsD["DALPHA0_DTAU_CONSTDELTA"] = ""
si_unitsD["dalpha0_dtau_constdelta"] = ""
si_unitsD["DALPHAR_DDELTA_CONSTTAU"] = ""
si_unitsD["dalphar_ddelta_consttau"] = ""
si_unitsD["DALPHAR_DTAU_CONSTDELTA"] = ""
si_unitsD["dalphar_dtau_constdelta"] = ""
si_unitsD["DBVIRIAL_DT"] = ""
si_unitsD["dBvirial_dT"] = ""
si_unitsD["DCVIRIAL_DT"] = ""
si_unitsD["dCvirial_dT"] = ""
si_unitsD["DELTA"] = ""
si_unitsD["Delta"] = ""
si_unitsD["DIPOLE_MOMENT"] = "C m"
si_unitsD["dipole_moment"] = "C m"
si_unitsD["DMASS"] = "kg/m^3"
si_unitsD["Dmass"] = "kg/m^3"
si_unitsD["DMOLAR"] = "mol/m^3"
si_unitsD["Dmolar"] = "mol/m^3"
si_unitsD["FH"] = ""
si_unitsD["FRACTION_MAX"] = ""
si_unitsD["fraction_max"] = ""
si_unitsD["FRACTION_MIN"] = ""
si_unitsD["fraction_min"] = ""
si_unitsD["FUNDAMENTAL_DERIVATIVE_OF_GAS_DYNAMICS"] = ""
si_unitsD["fundamental_derivative_of_gas_dynamics"] = ""
si_unitsD["G"] = "J/kg"
si_unitsD["GAS_CONSTANT"] = "J/mol/K"
si_unitsD["gas_constant"] = "J/mol/K"
si_unitsD["GMASS"] = "J/kg"
si_unitsD["Gmass"] = "J/kg"
si_unitsD["GMOLAR"] = "J/mol"
si_unitsD["Gmolar"] = "J/mol"
si_unitsD["GMOLAR_RESIDUAL"] = "J/mol/K"
si_unitsD["Gmolar_residual"] = "J/mol/K"
si_unitsD["GWP100"] = ""
si_unitsD["GWP20"] = ""
si_unitsD["GWP500"] = ""
si_unitsD["H"] = "J/kg"
si_unitsD["HELMHOLTZMASS"] = "J/kg"
si_unitsD["Helmholtzmass"] = "J/kg"
si_unitsD["HELMHOLTZMOLAR"] = "J/mol"
si_unitsD["Helmholtzmolar"] = "J/mol"
si_unitsD["HH"] = ""
si_unitsD["HMASS"] = "J/kg"
si_unitsD["Hmass"] = "J/kg"
si_unitsD["HMOLAR"] = "J/mol"
si_unitsD["Hmolar"] = "J/mol"
si_unitsD["HMOLAR_RESIDUAL"] = "J/mol/K"
si_unitsD["Hmolar_residual"] = "J/mol/K"
si_unitsD["I"] = "N/m"
si_unitsD["ISENTROPIC_EXPANSION_COEFFICIENT"] = ""
si_unitsD["isentropic_expansion_coefficient"] = ""
si_unitsD["ISOBARIC_EXPANSION_COEFFICIENT"] = "1/K"
si_unitsD["isobaric_expansion_coefficient"] = "1/K"
si_unitsD["ISOTHERMAL_COMPRESSIBILITY"] = "1/Pa"
si_unitsD["isothermal_compressibility"] = "1/Pa"
si_unitsD["L"] = "W/m/K"
si_unitsD["M"] = "kg/mol"
si_unitsD["MOLAR_MASS"] = "kg/mol"
si_unitsD["molar_mass"] = "kg/mol"
si_unitsD["MOLARMASS"] = "kg/mol"
si_unitsD["molarmass"] = "kg/mol"
si_unitsD["MOLEMASS"] = "kg/mol"
si_unitsD["molemass"] = "kg/mol"
si_unitsD["O"] = "J/kg/K"
si_unitsD["ODP"] = ""
si_unitsD["P"] = "Pa"
si_unitsD["P_CRITICAL"] = "Pa"
si_unitsD["p_critical"] = "Pa"
si_unitsD["P_MAX"] = "Pa"
si_unitsD["P_max"] = "Pa"
si_unitsD["P_MIN"] = "Pa"
si_unitsD["P_min"] = "Pa"
si_unitsD["P_REDUCING"] = "Pa"
si_unitsD["p_reducing"] = "Pa"
si_unitsD["P_TRIPLE"] = "Pa"
si_unitsD["p_triple"] = "Pa"
si_unitsD["PCRIT"] = "Pa"
si_unitsD["Pcrit"] = "Pa"
si_unitsD["pcrit"] = "Pa"
si_unitsD["PH"] = ""
si_unitsD["PHASE"] = ""
si_unitsD["Phase"] = ""
si_unitsD["PIP"] = ""
si_unitsD["PMAX"] = "Pa"
si_unitsD["pmax"] = "Pa"
si_unitsD["PMIN"] = "Pa"
si_unitsD["pmin"] = "Pa"
si_unitsD["PRANDTL"] = ""
si_unitsD["Prandtl"] = ""
si_unitsD["PTRIPLE"] = "Pa"
si_unitsD["ptriple"] = "Pa"
si_unitsD["Q"] = "mol/mol"
si_unitsD["RHOCRIT"] = "kg/m^3"
si_unitsD["rhocrit"] = "kg/m^3"
si_unitsD["RHOMASS_CRITICAL"] = "kg/m^3"
si_unitsD["rhomass_critical"] = "kg/m^3"
si_unitsD["RHOMASS_REDUCING"] = "kg/m^3"
si_unitsD["rhomass_reducing"] = "kg/m^3"
si_unitsD["RHOMOLAR_CRITICAL"] = "mol/m^3"
si_unitsD["rhomolar_critical"] = "mol/m^3"
si_unitsD["RHOMOLAR_REDUCING"] = "mol/m^3"
si_unitsD["rhomolar_reducing"] = "mol/m^3"
si_unitsD["S"] = "J/kg/K"
si_unitsD["SMASS"] = "J/kg/K"
si_unitsD["Smass"] = "J/kg/K"
si_unitsD["SMOLAR"] = "J/mol/K"
si_unitsD["Smolar"] = "J/mol/K"
si_unitsD["SMOLAR_RESIDUAL"] = "J/mol/K"
si_unitsD["Smolar_residual"] = "J/mol/K"
si_unitsD["SPEED_OF_SOUND"] = "m/s"
si_unitsD["speed_of_sound"] = "m/s"
si_unitsD["SURFACE_TENSION"] = "N/m"
si_unitsD["surface_tension"] = "N/m"
si_unitsD["T"] = "K"
si_unitsD["T_CRITICAL"] = "K"
si_unitsD["T_critical"] = "K"
si_unitsD["T_FREEZE"] = "K"
si_unitsD["T_freeze"] = "K"
si_unitsD["T_MAX"] = "K"
si_unitsD["T_max"] = "K"
si_unitsD["T_MIN"] = "K"
si_unitsD["T_min"] = "K"
si_unitsD["T_REDUCING"] = "K"
si_unitsD["T_reducing"] = "K"
si_unitsD["T_TRIPLE"] = "K"
si_unitsD["T_triple"] = "K"
si_unitsD["TAU"] = ""
si_unitsD["Tau"] = ""
si_unitsD["TCRIT"] = "K"
si_unitsD["Tcrit"] = "K"
si_unitsD["TMAX"] = "K"
si_unitsD["Tmax"] = "K"
si_unitsD["TMIN"] = "K"
si_unitsD["Tmin"] = "K"
si_unitsD["TTRIPLE"] = "K"
si_unitsD["Ttriple"] = "K"
si_unitsD["U"] = "J/kg"
si_unitsD["UMASS"] = "J/kg"
si_unitsD["Umass"] = "J/kg"
si_unitsD["UMOLAR"] = "J/mol"
si_unitsD["Umolar"] = "J/mol"
si_unitsD["V"] = "Pa s"
si_unitsD["VISCOSITY"] = "Pa s"
si_unitsD["viscosity"] = "Pa s"
si_unitsD["Z"] = ""




import re
import CoolProp.CoolProp as CP

incomp_pure_solnL = CP.get_global_param_string('incompressible_list_solution').split(',')

incomp_pure_fluidL = CP.get_global_param_string('incompressible_list_pure').split(',')


def parse_coolprop_mixture(mixture_name):
    """
    Parses a CoolProp mixture name to extract the base name and mass fraction percentage.

    Args:
        mixture_name (str): The CoolProp mixture name (e.g., "LiBr[0.23]" or "LiBr-23%").

    Returns:
        tuple: A tuple containing the base name (str) and mass fraction percentage (float).
               Returns (None, None) if parsing fails.
    """
    if mixture_name is None or not isinstance(mixture_name, str):
        return None, None
    
    # Pattern for just the base name, mass fraction is average of range
    if mixture_name in incomp_pure_solnL:

        frac_min = CP.PropsSI('fraction_min','INCOMP::%s'%mixture_name) 
        frac_max = CP.PropsSI('fraction_max','INCOMP::%s'%mixture_name) 

        pcent = int(100*(frac_min + frac_max)/2)
        print( 'Calculated Average Mass Fraction =', pcent )

        return mixture_name, pcent # for name w/o fraction_mass, assume average of range

    # Pattern for [fraction] notation
    match_bracket = re.match(r"([a-zA-Z0-9]+)\[([0-9.]+)\]", mixture_name)
    if match_bracket:
        base_name = match_bracket.group(1)
        mass_fraction_percentage = float(match_bracket.group(2)) * 100.0
        return base_name, mass_fraction_percentage

    # Pattern for -fraction% notation
    match_percent = re.match(r"([a-zA-Z0-9]+)-([0-9.]+)\%", mixture_name)
    if match_percent:
        base_name = match_percent.group(1)
        mass_fraction_percentage = float(match_percent.group(2))
        return base_name, mass_fraction_percentage

    return None, None  # Parsing failed

    # # Example usage:
    # print(parse_coolprop_mixture("LiBr[0.23]"))
    # print(parse_coolprop_mixture("LiBr-23%"))
    # print(parse_coolprop_mixture("Water"))
    # print(parse_coolprop_mixture("NaCl[0.1]"))
    # print(parse_coolprop_mixture("SomeInvalidString"))

def print_avoid_valerr():
    print( '.'*22, 'To avoid ValueError set "auto_fix_value_errors" to True', '.'*22 )

def format_float(float_val, output_len=9, sig_digits=4):
    """
    Formats a floating point number to a string of a specified length and significant digits.

    Parameters:
    float_val (float): The floating point number to format.
    output_len (int): The required length of the output string.
    sig_digits (int): The number of significant digits of float_val in the output string.

    Returns:
    str: The formatted string.
    """

    #  using %g format and cleaning up long exponents if "e" format
    format_str = f"%.{sig_digits}g"
    result = format_str % float_val
    result = result.replace('e+0', 'e').replace('e-0', 'e-').replace('e+', 'e')
    
    return result.ljust(output_len)

class Same_g_len:
    def __init__(self, obj, properties):
        """
        Initialize Same_g_len with an object and a list of property names.

        Parameters:
        obj (object): The object from which to get the property values.
        properties (list): A list of strings that are the names of the object's properties.
        
        Returns: properties padded with spaces to _max_length
        """
        self._max_length = 0
        self._properties = {}

        for prop in properties:
            if hasattr(obj, prop):
                value = getattr(obj, prop)
                try:
                    formatted_value = f"{value:g}"
                except:
                    formatted_value = str( value )
                self._properties[prop] = formatted_value
                self._max_length = max(self._max_length, len(formatted_value))

    def __getattr__(self, name):
        """
        Override __getattr__ to return right justified string of the maximum length.

        Parameters:
        name (str): The name of the attribute to access.

        Returns:
        str: The right justified string of the maximum length.
        """
        if name in self._properties:
            return self._properties[name].rjust(self._max_length)
        else:
            return '?'*self._max_length

def in_between(value, val1, val2):
    """
    Check if a value is between val1 and val2 inclusive, regardless of the order of val1 and val2.

    Parameters:
    value (float): The value to check.
    val1 (float): One end of the range.
    val2 (float): The other end of the range.

    Returns:
    bool: True if value is between val1 and val2 inclusive, False otherwise.
    """
    min_val = min(val1, val2)
    max_val = max(val1, val2)
    return min_val <= value <= max_val

def opposite_signs(x, y):
    """
    Check if two numbers have opposite signs.
    Zero is considered to have the opposite sign to a negative number.

    Parameters:
    x (float): The first number.
    y (float): The second number.

    Returns:
    bool: True if x and y have opposite signs, False otherwise.
    """
    return (x == 0 and y != 0) or (y == 0 and x != 0) or (x < 0 < y) or (y < 0 < x)

if __name__ == "__main__":

    # Example usage
    print( '... format_float ...')
    print('"%s"'%format_float(12345678.9))  # Output: "1.235e7  "
    print('"%s"'%format_float(0.00000123))  # Output: "1.23e-6  "
    print('"%s"'%format_float(-9876543.21)) # Output: "-9.877e6 "
    print('"%s"'%format_float(123.456))     # Output: "123.5    "
    print('"%s"'%format_float(0.123456))    # Output: "0.1235   "


    # Example usage
    print()
    print( '... in_between ...')
    print(in_between(5, 1, 10))  # Output: True
    print(in_between(5, 10, 1))  # Output: True
    print(in_between(1, 10, 1))  # Output: True
    print(in_between(10, 1, 10)) # Output: True
    print(in_between(0, 1, 10))  # Output: False
    print(in_between(15, 1, 10)) # Output: False

    # Example usage
    print()
    print( '... opposite_signs ...')
    # Example usage with expected answers
    def test_opposite_signs():
        assert opposite_signs(5, -10) == True, "Test case 1 failed"
        assert opposite_signs(-5, 10) == True, "Test case 2 failed"
        assert opposite_signs(0, -10) == True, "Test case 3 failed"
        assert opposite_signs(-10, 0) == True, "Test case 4 failed"
        assert opposite_signs(10, 0) == True, "Test case 5 failed"
        assert opposite_signs(5, 10)   == False, "Test case 6 failed"
        assert opposite_signs(-5, -10) == False, "Test case 7 failed"
        assert opposite_signs(0, 0)    == False, "Test case 8 failed"
        print("All test cases passed")

    # Run the tests
    test_opposite_signs()




from engcoolprop.ec_incomp_soln import EC_Incomp_Soln

# Create incompressible soln object at T=500 degR, P=500 psia and max pressure = 5000 psia
ec_soln = EC_Incomp_Soln(symbol="MEG-30%", T=500, P=500, Pmax=5000) # T=degR, P=psia

# OR... After ec_soln has been crated
# ec_soln.setTP( 500, 500)

# Print state point
ec_soln.printProps()



#!/usr/bin/env python
# -*- coding: ascii -*-


r"""
CoolProp has a set of incompressible solutions (e.g. Brines and Solutions)
EC_Incomp_Soln is a wrapper of those solutions using Engineering Units.

To get a list of these solutions use::

    incompressible_list_solution = CP.get_global_param_string('incompressible_list_solution')

EngCoolProp uses units of primarily inch, lbm, lbf, sec, BTU (some use of ft and hour).::

    #:c     The following are the default units for each property of Incompressible Fluids.
    #:c 
    #:c     T = Temperature = degR
    #:c     P = Pressure = psia
    #:c     D = Density = lbm/cu ft
    #:c     rho = Density = lbm/cu inch
    #:c     E = Internal Energy = BTU/lbm
    #:c     H = Enthalpy = BTU/lbm
    #:c     S = Entropy = BTU/lbm degR
    #:c     Cp = Heat Capacity (const. P) = BTU/lbm degR
    #:c     V = Viscosity = 1.0E5 * lbm/ft-sec
    #:c     C = Thermal Conductivity = BTU/ft-hr-R

"""
import traceback
from CoolProp.CoolProp import PropsSI
import CoolProp.CoolProp as CP

from engcoolprop.iteration_utils import calc_T_freeze
from engcoolprop.InterpProp_scipy import get_density_interpolator
from engcoolprop.ec_fluid import (CPeng_fromSI ,  CondEng_fromSI ,   DSI_fromEng ,  
                                  Deng_fromSI ,   PSI_fromEng, Peng_fromSI,
                                  SSI_fromEng ,  Seng_fromSI ,  TSI_fromEng ,  
                                  Teng_fromSI ,  UHSI_fromEng ,  UHeng_fromSI ,   Veng_fromSI )

from engcoolprop.safe_get_property import safe_get_INCOMP_prop, is_frac_max_check_inclusive
from engcoolprop.utils import Same_g_len, parse_coolprop_mixture, incomp_pure_solnL, print_avoid_valerr
from engcoolprop.iteration_utils import find_T_from_Dterp
from engcoolprop.banner import banner

# how to make a list of all incompressible solutions in coolprop
# incomp_pure_solnL = CP.get_global_param_string('incompressible_list_solution').split(',')


class EC_Incomp_Soln(object):
    
    def __init__(self,symbol="MEG-20%", T=None ,P=None, Pmax=10000.0,
                 show_warnings=2, auto_fix_value_errors=False):
        """        Init generic Incompressible Solution

        Args:
            symbol (str): Name and percent mass. ("LiBr[0.23]" or "LiBr-23%") Defaults to "MEG-20%".
            T (None or float): Temperature degR (None sets T to (Tmin+Tmax)/2)
            P (None or float): Pressure psia. (None set P to Pmax/10)
            Pmax (float): Max expected pressure psia. Defaults to 10000.0.
            show_warnings (int): Sets warning level(0=None, 1=Only serious, 2=all). Defaults to 2.
            auto_fix_value_errors (bool): Action when ValueError occurs. 
                                        True=correct problem with a warning, 
                                        False throws Exception Defaults to False.
        """
        self.auto_fix_value_errors = auto_fix_value_errors

        self.basename, self.percentage = parse_coolprop_mixture(symbol)
        if self.basename is None:
            raise ValueError( 'Input symbol "" not recognized.'%symbol )
        
        # If only basename was given, create a proper symbol with mass fraction
        if symbol == self.basename:
            symbol = symbol + '-' + '%g'%( self.percentage ) + '%'
        
        # print( "basename=%s, percentage=%s"%(self.basename, self.percentage) )

        frac_min = PropsSI('fraction_min','INCOMP::%s'%self.basename)
        frac_max = PropsSI('fraction_max','INCOMP::%s'%self.basename)
        # print( "CoolProp DB frac_min=%s, frac_max=%s"%(frac_min, frac_max) )

        frac_max_hi = frac_max  * 1.0000000000002 # needed to handle CoolProp non-inclusive range check


        self.pcent_min_str = '%g'%( frac_min*100 ) + '%'
        self.pcent_max_str = '%g'%( frac_max*100 ) + '%'

        self.fraction = self.percentage / 100.0
        # print( "self.fraction=%g"%self.fraction )

        # Make sure that solution base name is in CoolProp DB
        if self.basename not in incomp_pure_solnL:
            raise ValueError( '"%s" is NOT in coolprop incompressible list\n%s'%(symbol, repr(incomp_pure_solnL) ) )

        if show_warnings>1 and auto_fix_value_errors:
            banner( 'NOTICE: any input violations on limits of T, D or mass fraction\nwill be automatically corrected (set to min or max).'+\
                   '\nTo change this behavior set "auto_fix_value_errors" to False'+\
                    '\nTo suppress this banner set "show_warnings" to 0 or 1')


        # ==========  Account for errors in CoolProp DB ======================
        # Feb 2025 non-inclusive fluids ['ExampleSolution', 'IceEA', 'IceNA', 'IcePG', 'MAM2', 'MKA2', 
        #                                'MMG2', 'MPG2', 'VMG', 'ZLC', 'ZMC' ]
        frac_max_is_inclusive = is_frac_max_check_inclusive( self.basename )

        frac_max_adjusted = frac_max # might be changed if not frac_max_is_inclusive

        if not frac_max_is_inclusive:
            
            frac_max_adjusted = frac_max - 0.00001

            if show_warnings and self.fraction >= frac_max:
                print( 'WARNING: CoolProp has a round-off problem with frac_max=%g for %s'%(frac_max, self.basename) )
                print( "   CoolProp DB frac_min=%s, frac_max=%s"%(frac_min, frac_max) )
                print( '   Changing frac_max from %g to %g'%(frac_max, frac_max_adjusted))


        # make sure that fraction is in range
        if self.fraction < frac_min:
            if self.auto_fix_value_errors:
                new_pcent = '%g'%( frac_min*100 ) + '%'
                if show_warnings:
                    print( 'WARNING: input percentage(%s) is too low. Reset to %s'%( self.percentage, new_pcent ))
                self.percentage = new_pcent
                self.fraction = frac_min
                symbol = self.basename + '-' + new_pcent
            else:
                print_avoid_valerr()
                raise ValueError('Input percentage(%s) is too low. min percent = %g'%( self.percentage, frac_min*100 ))

        elif self.fraction >= frac_max_adjusted and self.fraction <= frac_max_hi: 
            # Always fix the CoolProp problem of non-inclusive range check
            # Need to reduce self.fraction below frac_max with non-inclusive range check.
            if not frac_max_is_inclusive:
                new_pcent = '%.3f'%( frac_max_adjusted*100 ) + '%'

                if show_warnings:
                    print( 'WARNING: input percentage(%s) is adjusted for CoolProp issue. Reset to %s'%( self.percentage, new_pcent ))
                self.percentage = new_pcent
                self.fraction = frac_max
                symbol = self.basename + '-' + new_pcent

        elif self.fraction > frac_max:
            if self.auto_fix_value_errors:
                # Need to reduce self.fraction below frac_max with non-inclusive range check.
                if not frac_max_is_inclusive:
                    new_pcent = '%.3f'%( frac_max_adjusted*100 ) + '%'
                else:
                    new_pcent = '%g'%( frac_max*100 ) + '%'

                if show_warnings:
                    print( 'WARNING: input percentage(%s) is too high. Reset to %s'%( self.percentage, new_pcent ))
                self.percentage = new_pcent
                self.fraction = frac_max
                symbol = self.basename + '-' + new_pcent
            else:
                print_avoid_valerr()
                raise ValueError( 'Input percentage(%s) is too high. max percent = %g'%( self.percentage, frac_max*100 ) )


        
        self.symbol = symbol


        self.Pmax = Pmax # highest pressure considered in any iterative calcs (can still input P > Pmax)
        self.Pmax_si = PSI_fromEng( Pmax )
        self.Pmin = 0

        if P is None:
            P = int( Pmax / 10 )
        self.P = P

        self.show_warnings = show_warnings # some calcs will issue warning statements if show_warnings >0 severe>1

        self.fluid = 'INCOMP::%s'%symbol

        # get temperature limits (Note attempt to avoid CoolProp round-off problems)
        self.Tmin_si =   PropsSI('Tmin','T',0,'P',0,'INCOMP::%s'%symbol) * 1.0000000000000002
        self.Tmax_si =   PropsSI('Tmax','T',0,'P',0,'INCOMP::%s'%symbol) * 0.9999999999999999

        # print( "self.Tmin_si=%g degK, self.Tmax_si=%g degK"%(self.Tmin_si, self.Tmax_si) )

        try:
            self.T_freeze_si = PropsSI('T_freeze','INCOMP::%s'%self.symbol)
            # print( "T_freeze = %.1f degR"%Teng_fromSI(self.T_freeze_si) )
            
            if self.T_freeze_si > self.Tmin_si:
                if self.show_warnings:
                    print( 'NOTICE: Tmin=%.1f degR has been increased to T_freeze + 1 = %.1f degR'%\
                          ( Teng_fromSI(self.Tmin_si), Teng_fromSI(self.T_freeze_si + 5.0/9.0)) )
                self.Tmin_si = self.T_freeze_si + 5.0/9.0
        except:
            # If no freezing point found, set it to 0
            self.T_freeze_si = 0

        if self.T_freeze_si <= 0:
            if self.show_warnings:
                print('NOTICE: T_freeze NOT in CoolProp DB. Trying to find it by traceback Exception.')
            self.T_freeze_si = calc_T_freeze(self)
            if self.T_freeze_si > self.Tmin_si:
                if self.show_warnings:
                    print( 'NOTICE: Tmin=%.1f degR has been increased to T_freeze + 1 = %.1f degR'%\
                          ( Teng_fromSI(self.Tmin_si), Teng_fromSI(self.T_freeze_si + 5.0/9.0)) )
                self.Tmin_si = self.T_freeze_si  + 5.0/9.0

        self.T_freeze = Teng_fromSI(self.T_freeze_si)
        # print( "self.T_freeze=%.1f"%self.T_freeze )



        self.Tmin =  Teng_fromSI( self.Tmin_si  )
        self.Tmax =  Teng_fromSI( self.Tmax_si )
        # print( "Tmin=%.1f, Tmax=%.1f"%(self.Tmin, self.Tmax) )

        self.Tmid = (self.Tmin + self.Tmax) / 2.0

        self.Dterp = get_density_interpolator( self )

        self.check_visc_support()

        if T is None:
            T = int((self.Tmin + self.Tmax) / 2.0)
            # else: T = input T

        # if input T is in range, use it... otherwise
        if T<self.Tmin:
            if self.auto_fix_value_errors:                
                self.T = self.Tmin + 0.001
                if show_warnings:
                    print( 'NOTICE: input T too low. Changed from: %g to: %g degR'%(T, self.T) )
                    print( "        INCOMP: Tmin=%g, Tmax=%g"%(self.Tmin, self.Tmax) )
            else:
                print_avoid_valerr()
                raise ValueError("Input T too low. T=%g,  Tmin=%g"%(T, self.Tmin))
                
        elif T>self.Tmax:
            if self.auto_fix_value_errors:                
                self.T = self.Tmax - 0.001
                if show_warnings:
                    print( 'NOTICE: input T too high. Changed from: %g to: %g degR'%(T, self.T) )
                    print( "        INCOMP: Tmin=%g, Tmax=%g"%(self.Tmin, self.Tmax) )
            else:
                print_avoid_valerr()
                raise ValueError("Input T too high. T=%g,  Tmax=%g"%(T, self.Tmax))
                
        else:
            self.T = T

        self.calc_min_max_props()

        # set properties to input T and P
        # print( 'First call to setTP')
        self.setTP(self.T, self.P)
        

        # Note that Psat is NOT supported
            
    def check_visc_support(self):
        try:
            _ =  PropsSI('V','T',self.Tmin_si,'P',self.Pmax_si, self.fluid ) # Vmax at Tmin
            _ =  PropsSI('V','T',self.Tmax_si,'P',self.Pmax_si, self.fluid ) # Vmin at Tmax
            self.visc_is_supported = True
        except:
            self.visc_is_supported = False
            if self.show_warnings:
                print( 'check_visc_support: Visc NOT supported for fluid:', self.symbol)

        # print( 'Found visc_is_supported =', self.visc_is_supported)


    def set_warnings(self, show_warnings=2):
        """Set the show_warnings flag"""
        if show_warnings is True:
            show_warnings = 1
        elif show_warnings is False:
            show_warnings = 0

        self.save_show_warnings = self.show_warnings
        self.show_warnings = show_warnings

    def pause_warnings(self):
        """Pause all warnings until self.show_warnings is changed"""
        self.save_show_warnings = self.show_warnings
        self.show_warnings = 0

    def restore_warnings(self):
        """Restore the self.show_warnings flag to the last pause_warnings condition"""
        try:
            self.show_warnings = self.save_show_warnings
        except:
            pass
    

    def getStrTransport(self):
        '''create a string from the Transport properties'''
        return  "%s Cp=%6g Visc=%6g ThCond=%6g" %\
        (self.fluid,self.Cp, self.Visc, self.Cond)

    def getStrTPD(self):
        '''create a string from the TPDEHS properties'''
        return  "%s T=%6.1f P=%6.1f D=%.4f E=%6.2f H=%6.2f S=%.3f" %\
        (self.fluid,self.T, self.P, self.D, self.E, self.H, self.S)

    # def getStrTPDphase(self):
    #     '''create a string from the TPDEHS properties'''
    #     return  "%s T=%6.1f P=%6.1f D=%.4f E=%6.2f H=%6.2f S=%.3f" %\
    #     (self.fluid,self.T, self.P, self.D, self.E, self.H, self.S)

    def printTPD(self):
        '''print a string from the TPDEHS properties'''
        print(self.getStrTPD())

    def printTransport(self):
        '''print a string of Transport properties'''
        print(self.getStrTransport())
        
    def get_D_at_T(self, T): # degR
        Psi = PSI_fromEng( 5000 )
        
        Tsi = TSI_fromEng( T )
        
        D = Deng_fromSI( PropsSI('D', 'T',Tsi,'P',Psi, self.fluid) )
        return D # lbm/cuft


    def setTP(self,T=530.0,P=1000.0):
        '''Calc props from T and P'''


        if T < self.Tmin:
            if self.auto_fix_value_errors:
                if self.show_warnings and self.Tmin - T > 0.01:
                    print( 'T too low in setTP. Changed T=%g to Tmin=%g'%( T, self.Tmin ))
                T = self.Tmin
            else:
                print_avoid_valerr()
                raise ValueError("Input T too low in setTP. T=%g,  Tmin=%g"%(T, self.Tmin))
        if T > self.Tmax:
            if self.auto_fix_value_errors:
                # if self.show_warnings and T - self.Tmax  > 0.01:
                if self.show_warnings and T > self.Tmax + 0.1 :
                    print( 'T too high in setTP. Changed T=%g to Tmax=%g'%( T, self.Tmax ))
                T = self.Tmax
            else:
                print_avoid_valerr()
                raise ValueError("Input T too high in setTP. T=%g,  Tmin=%g"%(T, self.Tmax))

        self.Pinput = P # save input P in case Psat changes it.

        if P > self.Pmax: # Not a bad infraction so just warn
            if self.show_warnings:
                print( 'P=%g is greater than Pmax=%g'%(P, self.Pmax))


        self.T = T
        self.P = P
        
        Tsi = TSI_fromEng( T )
        Psi = PSI_fromEng( P )
        def get_prop( prop_desc='H' ):
            try:
                prop, good_Psi = safe_get_INCOMP_prop( prop_desc, Psi_val=Psi, ind_name='T', ind_si_val=Tsi, 
                                                       symbol=self.symbol, show_warnings=self.show_warnings>1,
                                                       Pmax=self.Pmax )
                return prop
            except:
                return float('inf')

        # self.D = Deng_fromSI( get_prop('D') )
        self.D = self.get_D_at_T( self.T )

        self.rho = self.D / 1728.0

        self.E = UHeng_fromSI( get_prop('U') )
        self.H = UHeng_fromSI( get_prop('H') )
        self.S = Seng_fromSI( get_prop('S') )
        self.Cp = CPeng_fromSI( get_prop('C') )

        # Some INCOMP fluids have no viscosity data
        if self.visc_is_supported:
            self.Visc = Veng_fromSI( get_prop('V') ) * 1.0E5
        else:
            self.Visc = float('inf')

        self.Cond = CondEng_fromSI( get_prop('L') )
        
    def setPD(self,P=1000.0,D=0.01):
        '''
        Calc props from P and D
        NOTE: The pressure has NO EFFECT on calculated temperature for incompressible density.
        '''

        if D < self.Dmin:
            if self.auto_fix_value_errors:
                if self.show_warnings:
                    print( 'D too low in setDP. Changed D=%g to Dmin=%g'%( D, self.Dmin ))
                D = self.Dmin
            else:
                print_avoid_valerr()
                raise ValueError("Input D too low in setPD. D=%g,  Dmin=%g"%(D, self.Dmin))

        if D > self.Dmax:
            if self.auto_fix_value_errors:
                if self.show_warnings:
                    print( 'D too high in setDP. Changed D=%g to Dmax=%g'%( D, self.Dmax ))
                D = self.Dmax
            else:
                print_avoid_valerr()
                raise ValueError("Input D too high in setPD. D=%g,  Dmax=%g"%(D, self.Dmax))


        self.Pinput = P # save input P in case Psat changes it.
        # P = self.adjust_P_for_Psat( P, ind_param="D", ind_val=D )

        if P > self.Pinput:
            if self.show_warnings:
                print( 'P too low in setPD. Changed P=%g to Psat=%g'%( self.Pinput, P ))

        self.P = P 
        self.D = D

        self.T, err_flag = find_T_from_Dterp(self, D)
        if err_flag and self.show_warnings:
            print( 'WARNING: in setPD, find_T_from_Dterp has an error.')

        self.setTP( self.T, self.P )
        


    def printProps(self):
        '''print a multiline property summary with units'''

        # get formatted floats that are same length to help the look of table
        SGL = Same_g_len(self, ['Cond', "Cp", 'D', 'E', 'H', 'P', 'S', 'T', 'Visc', 'rho', 'percentage', 'T_freeze'] ) # , 'Tsat'


        print("State Point for fluid",self.fluid,"("+ self.symbol +")")
        print("T =%s"%SGL.T," degR,",    '                             Range(%8g - %8g) degR'%(self.Tmin, self.Tmax))

        print("P =%s"%SGL.P," psia",    '                              Range(%8g - %8g) psia'%(self.Pmin, self.Pmax))
        if self.Pinput != self.P:
            print("    Pinput =%8g"%self.Pinput, '(Adjusted due to Psat)')

        print("D =%s"%SGL.D," lbm/cuft",    '                          Range(%8g - %8g) lbm/cuft'%(self.Dmin, self.Dmax))

        print("E =%s"%SGL.E," BTU/lbm",    '                           Range(%8g - %8g) BTU/lbm'%(self.Emin, self.Emax))
        print("H =%s"%SGL.H," BTU/lbm",    '                           Range(%8g - %8g) BTU/lbm'%(self.Hmin, self.Hmax))
        print("S =%s"%SGL.S," BTU/lbm degR",    '                      Range(%.6f - %.6f) BTU/lbm degR'%(self.Smin, self.Smax))
        print("Cp=%s"%SGL.Cp," BTU/lbm degR",   '                      Range(%8g - %8g) BTU/lbm degR'%(self.Cpmin, self.Cpmax))

        if self.Visc < float('inf'):
            print("V =%s"%SGL.Visc," viscosity [1.0E5 * lbm/ft-sec]", '    Range(%8g - %8g)'%(self.Viscmin, self.Viscmax) )
        else:
            print("V =UNDEFINED","viscosity [1.0E5 * lbm/ft-sec]" )

        print("C =%s"%SGL.Cond," thermal conductivity [BTU/ft-hr-R]", 'Range(%8g - %8g)'%(self.Condmin, self.Condmax) )

        if self.T_freeze > 0:
            print( "    T_freeze = %g degR"%self.T_freeze)

        print("    rho      = %s"%SGL.rho," lbm/cuin",   '              Range(%.6f - %.6f) lbm/cuin'%(self.rho_min, self.rho_max))
        print("    mass%%    = %s"%SGL.percentage,"base mass percent", '      Range(%s - %s)'%(self.pcent_min_str, self.pcent_max_str))
        
        
        
    def calc_min_max_props(self, do_print=False):
        # print( '.......................Entered calc_min_max_props ..........................')
        T_save = self.T
        P_save = self.P

        self.psat_is_supported = False

        # self.Tsat_min = self.get_Tsat( self.Pmin )
        # self.Tsat_max = self.get_Tsat( self.Pmax )


        # make a list of the four corners of the T,P space
        tpL = [(self.Tmin, 0), (self.Tmin, self.Pmax), (self.Tmax, 0), (self.Tmax, self.Pmax)]

        resultD = {} # key:min/max name (e.g. Dmin, Cpmax), value:float value

        # set starting values for min/max of these properties
        for prop_name in [ 'D', 'E', 'H', 'S', 'Cp', 'Visc', 'Cond' ]:
            resultD[prop_name+'min'] = float('inf')
            resultD[prop_name+'max'] = float('-inf')

        self.pause_warnings()
        for T,P in tpL:
            self.setTP( T, P )

            for prop_name in [ 'D', 'E', 'H', 'S', 'Cp', 'Visc', 'Cond' ]:
                if getattr(self, prop_name) < resultD[prop_name+'min']:
                    resultD[prop_name+'min'] = getattr(self, prop_name)

                if getattr(self, prop_name) > resultD[prop_name+'max']:
                    resultD[prop_name+'max'] = getattr(self, prop_name)
        self.restore_warnings()

        # set min/max values found in resultD
        for prop_name, value in resultD.items():
            # print( 'setting ', prop_name, value)
            setattr( self, prop_name, value )

        self.rho_min = self.Dmin / 1728.0
        self.rho_max = self.Dmax / 1728.0
        
        self.Dmin_si = DSI_fromEng( self.Dmin )
        self.Dmax_si =  DSI_fromEng( self.Dmax )

        if do_print:
            minmaxL = ['Cond', "Cp", 'D', 'E', 'H', 'P', 'S', 'T', 'Visc']

            for name in minmaxL:
                max_name = name + 'max'
                print( '%10s'%max_name, '%9g'%getattr(self, max_name) )
                min_name = name + 'min'
                print( '%10s'%min_name, '%9g'%getattr(self, min_name) )
        # print( '.......................Finished calc_min_max_props ..........................')

        # Restore state to T and P from calling method
        self.setTP( T_save, P_save)        



def dev_tests():
    """
    === incompressible_list_solution ===
    AEG,AKF,AL,AN,APG,ExampleDigital,ExampleMelinder,ExampleSecCool,ExampleSolution,
    FRE,GKN,IceEA,IceNA,IcePG,LiBr,MAM,MAM2,MCA,MCA2,MEA,MEA2,MEG,MEG2,MGL,MGL2,
    MITSW,MKA,MKA2,MKC,MKC2,MKF,MLI,MMA,MMA2,MMG,MMG2,MNA,MNA2,MPG,MPG2,PK2,PKL,
    VCA,VKC,VMA,VMG,VNA,ZAC,ZFC,ZLC,ZM,ZMC
    """

    symbol = 'MEG-99%'    
    print( '='*22, "%s with %% too high"%symbol, '='*22 )
    C = EC_Incomp_Soln( symbol=symbol, T=None, P=None, 
                        auto_fix_value_errors=True, show_warnings=1 )
    print()

    symbol = 'IceNA-1%'
    print( '='*22, "%s with %% too low"%symbol, '='*22 )
    C = EC_Incomp_Soln( symbol=symbol, T=None, P=None, 
                        auto_fix_value_errors=True, show_warnings=1 )
    print()

    
    symbol = 'MEG-20%'
    print( '='*22, "%s at T=None P=None"%symbol, '='*22 )
    C = EC_Incomp_Soln( symbol=symbol, T=None, P=None )

    C.printTPD()
    C.printTransport()
    print()

    C.printProps()
    
    # C.setTP(T= (C.Tmin+C.Tmax)/2.0, P=100) # this temperature throws an exception ???

    
    symbol = 'MPG2-57%'
    print( '='*22, "%s Has frac_max round-off problem"%symbol, '='*22 )
    C = EC_Incomp_Soln( symbol=symbol, T=None, P=None )

    C.printTPD()
    C.printTransport()
    print()

    C.printProps()

    print( '='*22, "%s Check setPD"%symbol, '='*22 )
    C.setPD( C.P, C.D )
    C.printProps()


    banner( 'This should fail due to "auto_fix_value_errors = False"\nand max mass fraction = "MEG-60%"' )
    print( '='*22, "print full %s properties"%symbol, '='*22 )
    try:
        C_bad = EC_Incomp_Soln( symbol='MEG-99%', T=None, P=0 )
    except:
        tb_str = traceback.format_exc()
        print( tb_str.split('raise ValueError')[-1])
        # print( tb_str )
    

    banner( 'This should fail due to "auto_fix_value_errors = False"\nand T=100 below Tmin=428.68' )
    print( '='*22, "print full %s properties"%symbol, '='*22 )
    try:
        C_bad = EC_Incomp_Soln( symbol='MEG-50%', T=100, P=0 )
        C_bad.printTPD()
    except:
        tb_str = traceback.format_exc()
        print( tb_str.split('raise ValueError')[-1])
        # print( tb_str )


    print( '='*22, "%s Check omission of mass fraction", '='*22 )
    C = EC_Incomp_Soln( symbol='MEG', T=None, P=None )
    C.printProps()

    

if __name__ == '__main__':
    dev_tests()



from CoolProp.CoolProp import PropsSI
from engcoolprop.ec_fluid import ( Peng_fromSI, PropsSI , TSI_fromEng, PSI_fromEng, UHeng_fromSI,
                                   Teng_fromSI, Deng_fromSI,  )
from engcoolprop.find_exception_threshold import find_exception_limit

def run_experiments( symbol ):
    """Start iterating pressure values to find the reference H=0 point"""
    Tmin_si =   PropsSI('Tmin','T',0,'P',0,'INCOMP::%s'%symbol)  
    Tmax_si =   PropsSI('Tmax','T',0,'P',0,'INCOMP::%s'%symbol) 

    Tref_si = (Tmin_si + Tmax_si) / 2.0

    print( 'Tref = %g degR'%Teng_fromSI( Tref_si ), '  degK = %g'%Tref_si )

    # density is invariant with pressure for INCOMP
    Psi = PSI_fromEng( 1000.0 )
    Dmax_si =  PropsSI('D','T',Tmin_si,'P',Psi,'INCOMP::%s'%symbol) # Dmax at Tmin
    Dmin_si =  PropsSI('D','T',Tmax_si,'P',Psi,'INCOMP::%s'%symbol) # Dmin at Tmax

    Dmax = Deng_fromSI( Dmax_si )
    Dmin = Deng_fromSI( Dmin_si )
    print( 'Dmin = %g, Dmax = %g lbm/cuft'%(Dmin, Dmax) )

    # check reverse T calc
    Tmin_calc = Teng_fromSI( PropsSI('T','D',Dmax_si,'P',Psi, 'INCOMP::%s'%symbol ) )
    Tmax_calc = Teng_fromSI( PropsSI('T','D',Dmin_si,'P',Psi, 'INCOMP::%s'%symbol ) )

    print( 'Tmin = %g degR'%Teng_fromSI( Tmin_si ), '   Tmin_calc = %g'%Tmin_calc  )
    print( 'Tmax = %g degR'%Teng_fromSI( Tmax_si ), '   Tmin_calc = %g'%Tmax_calc  )

    Hmax = UHeng_fromSI( PropsSI('H','D',Dmin_si,'P',Psi, 'INCOMP::%s'%symbol ) )
    Hmin = UHeng_fromSI( PropsSI('H','D',Dmax_si,'P',0, 'INCOMP::%s'%symbol ) )
    print( '%s: '%symbol, 'Hmin = %g, Hmax = %g BTU/lbm'%(Hmin, Hmax) )

    Smax = UHeng_fromSI( PropsSI('S','D',Dmin_si,'P',Psi, 'INCOMP::%s'%symbol ) )
    Smin = UHeng_fromSI( PropsSI('S','D',Dmax_si,'P',Psi, 'INCOMP::%s'%symbol ) )
    print( '%s: '%symbol,  'Smin = %g, Smax = %g BTU/lbm degR'%(Smin, Smax) )
    
    print( '%s: '%symbol,  'Tmin_si = %g,  Tmax_si= %g'%( Tmin_si, Tmax_si) )
    t_range = Tmax_si - Tmin_si
    N = 9
    tL = [Tmin_si + (i/N)*t_range for i in range(N+1)]
    Psi2 = PSI_fromEng( 100.0 )
    for Tsi in tL:
        if 1:#try:
            Dsi =  PropsSI('D','T',Tsi,'P',Psi,'INCOMP::%s'%symbol)
            print(  '%s: '%symbol, 'D =', Deng_fromSI(Dsi) , 'lbm/cuft', '   at T = %g degK'%Tsi, ' = %g degR'%Teng_fromSI( Tsi )  )

            Hsi =  PropsSI('H','T',Tsi,'P',Psi,'INCOMP::%s'%symbol)
            print( '%s: '%symbol,  '    H =', UHeng_fromSI(Hsi) , 'BTU/lbm', '   at T = %g degK'%Tsi, ' = %g degR'%Teng_fromSI( Tsi )  )

            # Hsi2 =  PropsSI('H','T',Tsi,'P',Psi2,'INCOMP::%s'%symbol)
            # print( '    H =', UHeng_fromSI(Hsi2) , 'BTU/lbm', '   at T = %g degK'%Tsi, ' = %g degR'%Teng_fromSI( Tsi )  )

        # except:
        #     print( 'FAILED: P = %g'%P )

    def get_H( P, Tsi):
        Psi = PSI_fromEng( P )
        Hsi = PropsSI('H','T',Tsi,'P',Psi,'INCOMP::%s'%symbol)

    N = 9
    tL = [Tmin_si + (i/N)*t_range for i in range(N+1)]
    for Tsi in tL:
        f = lambda P: get_H( P, Tsi )
        good_limit_Psi = find_exception_limit(f, tolerance=1e-4, lower_bound=1.0e-9, upper_bound=10000)
        if good_limit_Psi is None:
            print( "==> FAILED:",  '%s: '%symbol, '   at T = %g degK'%Tsi, '(%g degR)'%Teng_fromSI( Tsi )  )
        else:
            print( '%s: '%symbol,  'good_limit_Psi = %8g Pa'%good_limit_Psi, '   at T = %g degK'%Tsi, '(%g degR)'%Teng_fromSI( Tsi )  )

if __name__ == "__main__":

    run_experiments( 'DowQ')



